/* Cable and Wireless Customization of Jabber Instant Messaging Server */

/* O.D.G. Griffiths     Started: 4-Sep-00 */

/* ATTENTION: Jabber Server Version: 1.4 */

/* Source file name:    cw_sysdef.h */


#ifndef CW_SYSDEF_H_INCLUDED
#define CW_SYSDEF_H_INCLUDED

#include <lber.h>
#include <ldap.h>

#include "jsm.h"


/*****************************************************/
/* System Definitions Specific to Cable and Wireless */
/*****************************************************/

/*** Constant Definitions ***/

#define VISPLISTXML	""
#define CWX500PWFLD	"imUserPassword"
#define LMT_THREADS     10


/*******************************************************/
/* Temporary Definitions for Testing with Test Harness */
/*******************************************************/

#if (1)
void log_error(char *zone, const char *msgfmt, ...);
#endif


/**********************************************/
/* Forward Declarations of Typedef References */
/**********************************************/

typedef struct waitevt_struct        _waitevt,       *waitevt;
typedef struct mod_ldap_conf_struct  _mod_ldap_conf, *mod_ldap_conf;


/********************************************************/
/* Definitions of Pointers to Functions for LDAP Access */
/********************************************************/

/* LMT callback simple function definition */

typedef result (*lmt_callback)( void* arg, LDAP* ld );


/**************************************/
/* Global Definitions for LDAP Access */
/**************************************/

typedef enum {A_PLAIN, A_DIGEST} authtype;


/*** Parameter Block for the LDAP Search Wait Event ***/

/* Used to pass data to and from the wait-event handler that
 * waits for the LDAP search to finish.
 */
struct waitevt_struct
{
    authtype            autype;     /* A_PLAIN or A_DIGEST */
    LDAP*               ld;
    int                 msgid;
    int                 rc;         /* Return code (Digest only) */
    LDAPMessage*        result;
};

/*** Struct in which to Store Global Configuration Data ***/

/* NOTE:  Pool-handling, vCard and Registration elements have been
 *        removed, as they are not required for authentication.
 *     *  Also, note the distinction between the LDAP (X.500)
 *        directory entry used to authorize the Jabber Server to
 *        access the LDAP directory ("mgrdn") and the search vector
 *        used to authenticate individual users (a combination of
 *        a string obtained from the Search Base lookup table and
 *        the element name supplied via "auth-attribute").
 *     *  Also note that the "base" string supplied here is now
 *        redundant, because the string is obtained from the table
 *        lookup instead.  However, it has been left in place for
 *        the moment because the configuration load function will
 *        fail without it.
 */
struct mod_ldap_conf_struct
{
    char*   host;          /* IP address of the LDAP dir. server */
    char*   port;          /* Port to use on the LDAP dir. server */
    char*   base;          /* Superseded by Search Base lookup */
    char*   mgrdn;         /* For LDAP dir. access authorization */
    char*   mgrpwd;        /* For LDAP dir. access authorization */
    char*   auth_attrib;   /* For LDAP dir. access authorization */
    char*   visplist;      /* Path to Search Base lookup table */
};


/*** Static Global Data for the LDAP Handler ***/

typedef struct cwldglob_struct
{
    mod_ldap_conf   cfg;
    instance        i;
}
*cwldglob, _cwldglob;


/*** Parameter Blocks for Thread Manager Callbacks ***/

/* LDAP queries are run on multiple threads so that they can run
 * concurrently.  The multiple thread system needs to cater for
 * different kinds of operation (password retrieval, password
 * changing, vCard data retrieval, etc.)  The thread manager
 * therefore only handles generalized thread activities and needs
 * to be passed a callback function containing the code that is
 * specific to the required action.  This callback function needs
 * to be passed all the data it will need to carry out the
 * required action. These parameter blocks pass the data needed
 * by each transaction type.
 */
typedef struct ldap_getpw_struct
{
    cwldglob  globdat;    /* Pointer to CW LDAP static global data */
    dpacket   dpkt;       /* Pointer to dpacket containing request */
    instance  inst;       /* Pointer to current Instance's ctl blk */
} _ldapgetpw, *ldapgetpw;


/**********************************************************/
/* Templates for Public Functions for LDAP Authentication */
/**********************************************************/

mod_ldap_conf  js_cw_ldap_config();

LDAP*  js_cw_ldap_init( mod_ldap_conf cfg );
void   js_cw_ldap_drop( mod_ldap_conf cfg, LDAP* ld );

result js_cw_checkUser( dpacket        dpkt,
                        LDAP*          ld,
                        mod_ldap_conf  cfg );

int    js_visplist_exist();
int    js_visplist_load( xmlnode cw_xdb_node, char* file );
char*  js_vispbase( char* vname );

char*  js_visp_s_to_l( char* sname );
char*  js_visp_l_to_s( char* lname );

void   ldap_threadman_init( mod_ldap_conf cfg );
void   ldap_threadman_send( pool p, lmt_callback f, void* arg );
void   ldap_threadman_shutdown( void *arg );


/**************************************************/
/* Templates for General Utility Public Functions */
/**************************************************/

void cw_cust_module_init( jsmi si );


/*** Debugging Display Functions used During Development ***/

void show_cfg( mod_ldap_conf cfg );

void show_uname_usage( int     sent_rcvd, 
                       char*   s_loc,
                       xmlnode n_top,
                       int     x_type );


#endif  /* CW_SYSDEF_H_INCLUDED */

